package com.example.demo.directions.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.department.service.DepartmentService;
import com.example.demo.directions.service.DirectionsService;
import com.example.demo.directions.model.DirectionsEntity;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/directions")
public class DirectionsController {
    private final DirectionsService directionsService;
    private final ModelMapper modelMapper;
    private final DepartmentService departmentService;

    public DirectionsController(DirectionsService directionsService, ModelMapper modelMapper,
            DepartmentService departmentService) {
        this.directionsService = directionsService;
        this.modelMapper = modelMapper;
        this.departmentService = departmentService;
    }

    private DirectionsDto toDto(DirectionsEntity entity) {
        return modelMapper.map(entity, DirectionsDto.class);
    }

    private DirectionsEntity toEntity(DirectionsDto dto) {
        final DirectionsEntity entity = modelMapper.map(dto, DirectionsEntity.class);
        entity.setDepartment(departmentService.get(dto.getDepartmentId()));
        return entity;
    }

    @GetMapping
    public List<DirectionsDto> getAll(@RequestParam(name = "departmentId", defaultValue = "0") Long departmentId) {
        return directionsService.getAll(departmentId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public DirectionsDto get(@PathVariable(name = "id") Long id) {
        return toDto(directionsService.get(id));
    }

    @PostMapping
    public DirectionsDto create(@RequestBody @Valid DirectionsDto dto) {
        return toDto(directionsService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public DirectionsDto update(@PathVariable(name = "id") Long id, @RequestBody DirectionsDto dto) {
        return toDto(directionsService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public DirectionsDto delete(@PathVariable(name = "id") Long id) {
        return toDto(directionsService.delete(id));
    }
}
