package com.example.demo.entrysData.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.example.demo.core.configuration.Constants;
import com.example.demo.department.service.DepartmentService;
import com.example.demo.entrysData.model.EntrysDataEntity;
import com.example.demo.entrysData.service.EntrysDataService;

import jakarta.validation.Valid;

@RestController
@RequestMapping(Constants.API_URL + "/entrysData")
public class EntrysDataController {
    private final EntrysDataService entrysDataService;
    private final ModelMapper modelMapper;
    private final DepartmentService departmentService;

    public EntrysDataController(EntrysDataService entrysDataService, ModelMapper modelMapper,
            DepartmentService departmentService) {
        this.entrysDataService = entrysDataService;
        this.modelMapper = modelMapper;
        this.departmentService = departmentService;
    }

    private EntrysDataDto toDto(EntrysDataEntity entity) {
        return modelMapper.map(entity, EntrysDataDto.class);
    }

    private EntrysDataEntity toEntity(EntrysDataDto dto) {
        final EntrysDataEntity entity = modelMapper.map(dto, EntrysDataEntity.class);
        entity.setDepartment(departmentService.get(dto.getDepartmentId()));
        return entity;
    }

    @GetMapping
    public List<EntrysDataDto> getAll(@RequestParam(name = "departmentId", defaultValue = "0") Long departmentId) {
        return entrysDataService.getAll(departmentId).stream().map(this::toDto).toList();
    }

    @GetMapping("/{id}")
    public EntrysDataDto get(@PathVariable(name = "id") Long id) {
        return toDto(entrysDataService.get(id));
    }

    @PostMapping
    public EntrysDataDto create(@RequestBody @Valid EntrysDataDto dto) {
        return toDto(entrysDataService.create(toEntity(dto)));
    }

    @PutMapping("/{id}")
    public EntrysDataDto update(@PathVariable(name = "id") Long id, @RequestBody EntrysDataDto dto) {
        return toDto(entrysDataService.update(id, toEntity(dto)));
    }

    @DeleteMapping("/{id}")
    public EntrysDataDto delete(@PathVariable(name = "id") Long id) {
        return toDto(entrysDataService.delete(id));
    }

    @PutMapping("/{id}/password")
    public EntrysDataDto updatePassword(@PathVariable(name = "id") Long id,
            @RequestParam(name = "newPassword") String newPas,
            @RequestParam(name = "oldPassword") String oldPas) {
        return toDto(entrysDataService.updatePassword(id, newPas, oldPas));
    }
}
